/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


var EXPORTED_SYMBOLS =
[
    "FolderEntry",
    "recentEntry",
    "separatorEntry"
];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://savefileto/common.jsm");


const recentEntry = {type: "recent"};
const separatorEntry = {type: "separator"};


function FolderEntry(entry, compare, filter)
{
    this.type = (entry.menu ? "menu" : "item");
    this.label = entry.label;
    this.labelLower = entry.label.toLowerCase();
    this.path = entry.path;
    this.pathLabel = (entry.pathLabel ? entry.pathLabel : entry.label);
    this._file = entry.file;
    
    this.compare = compare;
    this.filter = filter;
}

FolderEntry.prototype =
{
    get file()
    {
        if (! this._file)
        {
            var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
            file.initWithPath(this.path);
            this._file = file;
        }
        
        return this._file;
    },
    
    get hidden()
    {
        var file = this.file;
        return (! (file.exists() && file.isDirectory()));
    },
    
    get hasChildren()
    {
        return ((this.type == "menu") && this._getChildren(true));
    },
    
    get children()
    {
        return ((this.type == "menu") ? this._getChildren(false) : []);
    },
    
    _getChildren: function FE__getChildren(searchOnly)
    {
        var children = [];
        
        var pathLabel = this.pathLabel;
        var compare = this.compare;
        var filter = this.filter;
        
        var fileEnumerator = this.file.directoryEntries;
        while (fileEnumerator.hasMoreElements())
        {
            var file = fileEnumerator.getNext().QueryInterface(Ci.nsILocalFile);
            
            if (! file.isDirectory())
                continue;
            if (filter && filter(file))
                continue;
            if (searchOnly)
                return true;
            
            var entry =
            {
                menu: true,
                label: file.leafName,
                path: file.path,
                pathLabel: pathLabel + PATH_DELIM + file.leafName,
                file: file
            };
            
            entry = new FolderEntry(entry, compare, filter);
            children.push(entry);
        }
        
        if (searchOnly)
            return false;
        
        if (compare)
            children.sort(compare);
        
        return children;
    },
    
    compareTo: function FE_compareTo(entry)
    {
        var a = this.labelLower;
        var b = entry.labelLower;
        
        return ((a < b) ? -1 : ((a == b) ? 0 : 1));
    },
    
    equals: function FE_equals(entry)
    {
        return ((entry.type == this.type) && (entry.labelLower == this.labelLower) && (entry.path == this.path));
    },
    
    toString: function FE_toString()
    {
        return "{type: '" + this.type + "', label: '" + this.label + "', path: '" + this.path + "'}";
    }
}
